/*

  libkohn_rtsp - Copyright 2011-2014 by Michael Kohn
  http://www.mikekohn.net/
  mike@mikekohn.net

  Source distributed under the LGPL license.

*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>

#include "kohn_rtsp.h"

struct _rtsp_info
{
  int rtp_count;
  int rtcp_count;
  int rtp_sockfd;
  int rtcp_sockfd;
  struct sockaddr_in rtp_src_addr;
  struct sockaddr_in rtp_dst_addr;
  struct sockaddr_in rtcp_src_addr;
  struct sockaddr_in rtcp_dst_addr;
};

static int running=1;

void handler(int signal_num)
{
  printf("Signal caught\n");
  running=0;
}

void rtp_callback(unsigned char *data, int data_len, void *context)
{
struct _rtsp_info *rtsp_info=(struct _rtsp_info *)context;

#ifdef DEBUG
  printf("--- Data on channel %p of len %d  socket %d\n", context, data_len, rtsp_info->rtp_sockfd);
#endif

#ifdef SUPER_DEBUG
  int n;
  for (n=0; n<data_len; n++)
  {
    if ((n%16)==0) printf("\n");
    printf("%02x ", data[n]);
  }
  printf("\n");
#endif

  sendto(rtsp_info->rtp_sockfd, data, data_len, 0, (struct sockaddr *)&rtsp_info->rtp_dst_addr, sizeof(rtsp_info->rtp_dst_addr));

  rtsp_info->rtp_count++;
}

void rtcp_callback(unsigned char *data, int data_len, void *context)
{
struct _rtsp_info *rtsp_info=(struct _rtsp_info *)context;

#ifdef DEBUG
  printf("--- RTCP on channel %p of len %d\n", context, data_len);
#endif

#ifdef SUPER_DEBUG
  int n;
  for (n=0; n<data_len; n++)
  {
    if ((n%16)==0) printf("\n");
    printf("%02x ", data[n]);
  }
  printf("\n");
#endif

  sendto(rtsp_info->rtcp_sockfd, data, data_len, 0, (struct sockaddr *)&rtsp_info->rtcp_dst_addr, sizeof(rtsp_info->rtcp_dst_addr));

  rtsp_info->rtcp_count++;
}

int main(int argc, char *argv[])
{
struct _rtsp_info rtsp_info;
struct _kohn_rtsp *kohn_rtsp;
int port_start;

  if (argc!=4)
  {
    printf("redirect_rtsp - Copyright 2011-2014 by Michael Kohn\n");
    printf("Version: 2014-01-08\n");
    printf("Usage: %s <URL> <video_server> <startport>\n", argv[0]);
    exit(0);
  }

  rtsp_info.rtp_count=0;
  rtsp_info.rtcp_count=0;
  port_start=atoi(argv[3]);

  if (port_start<8000 || port_start>20000)
  {
    printf("Invalid port number.  Starting port should be between 8000 and 20000\n");
    exit(1);
  }

  printf(" RTP port: %d\n", port_start);
  printf("RTCP port: %d\n\n", port_start+1);

  rtsp_info.rtp_sockfd=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if (rtsp_info.rtp_sockfd<0)
  {
    printf("Can't open socket.\n");
    exit(1);
  }
  rtsp_info.rtcp_sockfd=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if (rtsp_info.rtcp_sockfd<0)
  {
    printf("Can't open socket.\n");
    exit(1);
  }

  memset(&rtsp_info.rtp_src_addr, 0, sizeof(rtsp_info.rtp_src_addr));
  rtsp_info.rtp_src_addr.sin_family=AF_INET;
  rtsp_info.rtp_src_addr.sin_addr.s_addr=htonl(INADDR_ANY);

  memset(&rtsp_info.rtcp_src_addr, 0, sizeof(rtsp_info.rtcp_src_addr));
  rtsp_info.rtcp_src_addr.sin_family=AF_INET;
  rtsp_info.rtcp_src_addr.sin_addr.s_addr=htonl(INADDR_ANY);

  memset(&rtsp_info.rtp_dst_addr, 0, sizeof(rtsp_info.rtp_dst_addr));
  rtsp_info.rtp_dst_addr.sin_family=AF_INET;

  struct hostent *host=NULL;

  if (inet_aton(argv[2], &rtsp_info.rtp_dst_addr.sin_addr)==0)
  {
    host=gethostbyname(argv[2]);
    if (host==NULL)
    {
      printf("Bad destination address\n");
      exit(1);
    }
    memcpy(&rtsp_info.rtp_dst_addr.sin_addr, host->h_addr_list[0], host->h_length);
  }
  rtsp_info.rtp_dst_addr.sin_port=htons(port_start);

  memset(&rtsp_info.rtcp_dst_addr, 0, sizeof(rtsp_info.rtcp_dst_addr));
  rtsp_info.rtcp_dst_addr.sin_family=AF_INET;

  if (inet_aton(argv[2], &rtsp_info.rtcp_dst_addr.sin_addr)==0)
  {
/*
    host=gethostbyname(argv[2]);
    if (host==NULL)
    {
      printf("Bad destination address\n");
      exit(1);
    }
*/
    memcpy(&rtsp_info.rtcp_dst_addr.sin_addr, host->h_addr_list[0], host->h_length);
  }
  rtsp_info.rtcp_dst_addr.sin_port=htons(port_start+1);

  int status=kohn_rtsp_open(&kohn_rtsp, argv[1], rtp_callback, rtcp_callback, KOHN_RTP_MODE_RTP, &rtsp_info);
  if (status!=0)
  {
    printf("Error %d opening stream\n", status);
    exit(1);
  }

  signal(SIGTERM,handler);
  signal(SIGINT,handler);

#ifdef DEBUG
  printf("Opened rtsp stream on channel %p\n", &rtsp_info);
#endif

  int count=1800;
  while(count>0 && running==1)
  {
    if (kohn_rtsp_is_running(kohn_rtsp)==0) break;
    count--;
    sleep(1);
  }

  printf("Stopping\n");
  kohn_rtsp_stop(kohn_rtsp);
  printf("Stopped\n");

#ifdef DEBUG
  printf("     Size: %dx%d\n", kohn_rtsp_get_width(kohn_rtsp), kohn_rtsp_get_height(kohn_rtsp));
  printf("Framerate: %f\n", kohn_rtsp_get_framerate(kohn_rtsp));
  printf("    Codec: %s\n", kohn_rtsp_get_codec(kohn_rtsp));
#endif

  int n=5;
  while(n>=0)
  {
    // We can send RTCP packets in here if we need to
    if (kohn_rtsp_is_running(kohn_rtsp)==0) break;
    sleep(1);
  }

  printf("Received %d rtp packets and %d rtcp packets.\n", rtsp_info.rtp_count, rtsp_info.rtcp_count);

  kohn_rtsp_close(kohn_rtsp);

  return 0;
}



